%% BMWStepResponse
% A simple model, assuming level road, is:
%    dv/dt = (Fnet - cD*v^2 - Froll)/m := f(v,Fnet)

%% Model Parameters
m = 2085;       % Vehicle mass, kg
cD = 0.4;       % Drag coefficient, N*sec^2/m^2
Froll = 228;    % Rolling resistance, N

%% Equilibrium Points
% Solve the following for an equilibrium point:
%    0 = Fnet - cD*v^2 - Froll
% Note: 1m/s ~= 2.24 mph.  
Fnet_eq = 400;
v_eq = sqrt( (Fnet_eq-Froll)/cD );

%% Linearization
% Taylor series expansion of f(v,Fnet) around the eq point gives:
%   d(delta_v)/dt ~=  [(-2*cD*veq)*delta_v + delta_Fnet]/m
% where the deviations are defined as:
%   delta_v = v-veq
%   delta_Fnet = Fnet-Fneteq
% This yields the linear ODE:
%   d(delta_v)/dt + a0*delta_v = b0*delta_Fnet
a0 = 2*cD*v_eq/m;
b0 = 1/m;
G = tf(b0,[1 a0]);

%% Simulations: Step Response

% Step response: Exact solution
Tf = 400;
t = linspace(0,Tf,1000);
dF = 50;               % Step input force, N
dv_part = b0*dF/a0;    % Particular solution, m/s
dv = dv_part*(1-exp(-a0*t));   % Velocity deviation from trim, m/s

% Note: The solution can also be approximately solved via numerical 
% integration using the STEP command
%    [dv_unit,t]= step(G,Tf);
%    dv = dv_unit*dF;

% Plot Results
figure(1); clf
plot(t,dv);
grid on;
xlabel('Time (sec)');
ylabel('Velocity Deviation, \delta_v (m/s)');
set(gca, 'LooseInset', get(gca, 'TightInset'));
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure
    myprint(1,'BMWStepResponseDev')
end


figure(2); clf
plot(t,dv+v_eq);
grid on;
xlabel('Time (sec)');
ylabel('Velocity, v (m/s)');
set(gca, 'LooseInset', get(gca, 'TightInset'));
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure
    myprint(2,'BMWStepResponseAct')
end

